﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.ComponentModel;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Runtime.InteropServices;
using System.Security.Cryptography;

namespace MyDNSUpdate
{
    public class Settings
    {
        public string IPURL { get; set; }
        public XmlTimeSpan IPInterval { get; set; }
        public XmlTimeSpan RetryInterval { get; set; }
        public BindingList<AccountSettings> Accounts { get; set; }
        private static bool cipherError = false;

        static StreamReader sr = null;

        public Settings()
        {
            IPURL = "http://checkip.dyndns.org/";
            IPInterval = TimeSpan.FromMinutes(10);
            RetryInterval = TimeSpan.FromMinutes(10);
            Accounts = new BindingList<AccountSettings>();
        }

        public static bool LoadSettings(ref Settings setting_I, out string log)
        {
            bool ret = true;
            log = "";

            System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(Settings));
            try {
                string xmlPath = Path.ChangeExtension(Assembly.GetEntryAssembly().Location, "xml");
                if (sr != null) {
                    sr.Close();
                    sr.Dispose();
                    sr = null;
                }
                FileStream fs = new FileStream(xmlPath, FileMode.Open, FileAccess.Read, FileShare.None);
                sr = new StreamReader(fs, new System.Text.UTF8Encoding(false));
                setting_I = (Settings)serializer.Deserialize(sr);
            }
            catch (FileNotFoundException) {
                ret = true; // ない場合はアカウント数で判別(サービスなら中止)
            }
            catch (IOException e) {
                const long ERROR_SHARING_VIOLATION = 0x20;
                const long ERROR_LOCK_VIOLATION = 0x21;
                long win32ErrorCode = Marshal.GetHRForException(e) & 0xFFFF;
                if ((win32ErrorCode == ERROR_SHARING_VIOLATION) || (win32ErrorCode == ERROR_LOCK_VIOLATION)) {
                    log = "Preference file access violation.";
                }
                else log = e.Message;
                ret = false;
            }
            catch (Exception e) {
                ret = false;
                if (e.InnerException.GetType() == typeof(CryptographicException)) {
                    log = "Cipher text in preference file is not valid.";
                    cipherError = true;
                }
                else {
                    log = e.Message;
                }
            }
            return ret;
        }

        private static bool CompareArray(Array a, Array b)
        {
            if (object.ReferenceEquals(a, b)) return true;

            if (a.Length != b.Length) return false;

            for (int i = 0; i < a.Length; i++) {
                if (!a.GetValue(i).Equals(b.GetValue(i))) {
                    return false;
                }
            }
            return true;
        }

        public static void SaveSettings(Settings setting_I)
        {
            string dummy;
            MemoryStream ms0 = null;
            MemoryStream ms1 = null;

            System.Xml.Serialization.XmlSerializer serializer1 = new System.Xml.Serialization.XmlSerializer(typeof(Settings));

            Settings tmp_settings = new Settings();
            CryptString.IgnoreCipherError = true;
            LoadSettings(ref tmp_settings, out dummy);

            if (!cipherError) {
                ms0 = new MemoryStream();
                ms1 = new MemoryStream();
                CryptString.Encryption = false;
                serializer1.Serialize(ms0, tmp_settings);
                serializer1.Serialize(ms1, setting_I);
            }
            if ((cipherError || !CompareArray(ms0.ToArray(), ms1.ToArray()))) {
                CryptString.Encryption = true;
                System.IO.StreamWriter sw = null;
                try {
                    string xmlPath = Path.ChangeExtension(Assembly.GetEntryAssembly().Location, "xml");
                    if (sr != null) {
                        sr.Close();
                        sr.Dispose();
                        sr = null;
                    }
                    sw = new System.IO.StreamWriter(xmlPath, false, new System.Text.UTF8Encoding(false));
                    serializer1.Serialize(sw, cipherError ? tmp_settings : setting_I);
                    sw.Close();
                }
                catch {
                }
                finally {
                    if (sw != null) sw.Close();
                }
            }
            if (ms0 != null) ms0.Close();
            if (ms1 != null) ms1.Close();
        }
    }

    public class AccountSettings
    {
        public bool Enabled { get; set; }
        public string Title { get; set; }
        public XmlTimeSpan Interval { get; set; }
        [XmlIgnore] public DateTime LastUpdate { get; set; }
        [XmlIgnore] public bool LastStatus { get; set; }
        public DetailSettings detail { get; set; }

        public AccountSettings()
        {
            Enabled = false;
            Title = "";
            Interval = new TimeSpan(7, 0, 0, 0);
            LastUpdate = DateTime.MinValue;
            LastStatus = false;
            detail = new DetailSettings();
        }
    }

    public class DetailSettings
    {
        public string URL { get; set; }
        public string OK { get; set; }
        public bool BasicAuth { get; set; }
        public CryptString Username { get; set; }
        public CryptString Password { get; set; }
        public BindingList<Parameters> HTTPParams { get; set; }

        public DetailSettings()
        {
            URL = "";
            BasicAuth = false;
            Username = "";
            Password = "";
            HTTPParams = new BindingList<Parameters>();
        }
    }

    public class Parameters
    {
        public string Name { get; set; }
        public CryptString Value { get; set; }

        public Parameters()
        {
            Name = "";
            Value = "";
        }
    }
}

